(* Copyright 2008 Christian Sternagel *)

(* This file is part of LIPS.

  LIPS is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as 
	published by the Free Software Foundation, either version 3 of 
	the License, or (at your option) any later version.

  LIPS is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public 
	License along with LIPS.  If not, see 
	<http://www.gnu.org/licenses/>. *)
	
type 'a t = 'a list;;

let hd = function
 | x :: _ -> x
 | _ -> failwith "Lst.hd: empty list"
;;
let tl = function
 | _ :: xs -> xs
 | _ -> failwith "Lst.tl: empty list"
;;
let rec append xs ys = if xs = [] then ys else hd xs :: append (tl xs) ys;;
let rec init i l = if l <= 0 then [] else i :: init i (l - 1);;
let rec take n xs =
 if n <= 0 then [] else match xs with
  | [] -> []
  | x :: xs -> x :: take (n - 1) xs
;;
let rec drop n xs =
 if n <= 0 then xs else match xs with
  | [] -> []
  | _ :: xs -> drop (n - 1) xs
;;
let split_at n xs = (take n xs, drop n xs);;
let rec fold f b = function
 | [] -> b
 | x :: xs -> f x (fold f b xs)
;;
let rec fold_left f b = function
 | [] -> b
 | x :: xs -> fold_left f (f b x) xs
;;
let rec filter p = function
 | [] -> []
 | x :: xs -> if p x then x :: filter p xs else filter p xs
;;
let length xs = fold (fun _ x -> x + 1) 0 xs;;
let map f xs = fold (fun x xs -> f x :: xs) [] xs;;
let rec last = function
 | [x] -> x
 | _ :: xs -> last xs
 | _ -> failwith "Lst.last: empty list"
;;
let rec nth n = function
 | [] -> failwith "Lst.nth: out of bounds"
 | x :: xs -> if n = 0 then x else nth (n - 1) xs
;;
let rec zip2 xs ys = match (xs, ys) with
 | (x :: xs, y :: ys) -> (x, y) :: zip2 xs ys
 | _ -> []
;;
let zip2_with f xs ys = map (fun (x, y) -> f x y) (zip2 xs ys);;
let rec fold1 f = function
 | [x] -> x
 | x :: xs -> f x (fold1 f xs)
 | _ -> failwith "Lst.fold1: empty list"
;;
let rec fold_left1 f = function
 | [] -> failwith "Lst.fold_left1: empty list"
 | x :: xs -> fold_left f x xs
;;
let rec zip4 ws xs ys zs = match (ws, xs, ys, zs) with
 | (w :: ws, x :: xs, y :: ys, z :: zs) -> (w, x, y, z) :: zip4 ws xs ys zs
 | _ -> []
;;
let rec take_while p = function
 | [] -> []
 | x :: xs -> if p x then x :: take_while p xs else []
;;
let rec drop_while p = function
 | [] -> []
 | x :: xs as ys -> if p x then drop_while p xs else ys
;;
let span p xs = (take_while p xs, drop_while p xs);;
let rev xs =
 let rec rev acc = function
  | [] -> acc
  | x :: xs -> rev (x :: acc) xs
 in
 rev [] xs
;;
let rec until p f x = if p x then x else until p f (f x);;
let concat xs = fold append [] xs;;




(** sort using mergesort *)
let rec sort compare = 
	let split l =
	  let rec split_aux l left right = 
	    match l,left,right with
	    | ([] | [_]),_,_ -> (List.rev left),right
	    | (_::_::t),_,h::right_t -> split_aux t (h::left) right_t
	    | _ -> assert false
  in split_aux l [] l  in
	let rec merge compare l1 l2 =
	  match l1,l2 with
	  | [],l | l,[] -> l
	  | h1::t1,h2::t2 ->
	    if compare h1 h2 < 1 then h1::(merge compare t1 l2)
	    else h2::(merge compare l1 t2)	
	in function
  | ([] | [_]) as l -> l
  | l ->  let left,right = split l in 
          merge compare (sort compare left) (sort compare right);;

let rec mem y = function
	| [] -> false
	| x::xs -> if x = y then true else mem y xs;;
